@@ -1,10 +1,10 @@
 ;;;-*- mode: lisp-interaction; syntax: elisp -*-;;;
 ;;
-;; "predoc.el" is a WYSIWYG majar mode for Predoc document format.
+;; "predoc-mode.el" is a WYSIWYG majar mode for Predoc document format.
 ;;
 ;;   Copyright (C) 2007 Kiyoka Nishiyama
 ;;
-;;     $Date: 2007/02/18 13:52:34 $
+;;     $Date: 2007/03/24 15:25:23 $
 ;;
 ;; This file is part of Predoc
 ;;
@@ -28,6 +28,11 @@
 ;;
 ;;
 ;; ChangeLog:
+;;   [0.9.2]
+;;     1. Supported `img' tag which recognize "http://" URL.
+;;     2. Supported "path/of/image/file.(jpg|png|bmp ... )" string convert into `img' tag.
+;;     3. Supported "http://host/path/of/contents... anchor-keyword" string convert into `a' tag.
+;;
 ;;   [0.9.1]
 ;;     1. Supported image resizing. ( width and height parameter of img tag )
 ;;     2. Supported predoc-insert-images. ( C-c l )
@@ -39,7 +44,7 @@
 ;;
 (require 'sgml-mode)
 
-(defconst predoc-version "0.9.1")
+(defconst predoc-version "0.9.2")
 
 (defconst predoc-alink-face  'link)
 (defconst predoc-entity-face 'highlight)
@@ -116,39 +121,65 @@ Buffer string between BEG and END are replaced with IMAGE."
 				     'invisible nil)))
 
 
-(defun predoc-create-image (file &optional width height)
-  (let (data pixel-width pixel-height)
-    (when (file-readable-p file)
-      ;;	(setq pixel-width (predoc-scale-pixel-width scale))
+
+;; insert image data to current-buffer.
+;;  e.x.)
+;;     (predoc-insert-image-data "http://www.sumibi.org/sumibi/sumibi_picture.png")
+(defun predoc-insert-image-data (url)
+  (cond 
+   ((string-match "http://" url)
+    (call-process "/usr/bin/curl"
+		  nil
+		  '(t nil)
+		  nil
+		  url))
+   (t
+    (insert-file-contents url))))
+
+
+(defun predoc-create-image (url &optional width height)
+  (let (data pixel-width pixel-height
+	     (m (string-match "http://" url)))
+    (when (or (file-readable-p url) m)
+      (when (not m)
+	(setq url (predoc-expand-full-path url)))
       (setq pixel-width (or width
 			    ""))
       (setq pixel-height (or height
 			     ""))
-      (when (or width height)
-	(if (not (file-executable-p predoc-convert-program))
-	    (message (format "'%s' does not executable... Image can't be convert size." predoc-convert-program))
+      (if (not (file-executable-p predoc-convert-program))
+	  (predoc-warning "'%s' does not executable... Image can't be convert size." predoc-convert-program)
+	(progn
 	  (setq data
 		(with-temp-buffer
 		  (let ((coding-system-for-read 'binary)
 			(coding-system-for-write 'binary)
 			(auto-image-file-mode nil))
 		    (set-buffer-multibyte nil)
-		    (insert-file-contents file)
+		    (predoc-insert-image-data url)
 		    ;; (message (format "1: min = %s  max = %s " (point-min) (point-max)))
-		    (call-process-region (point-min) (point-max)
-					 predoc-convert-program
-					 t 
-					 '(t nil)
-					 nil
-					 "-" 
-					 "-resize"
-					 (format "%sx%s" pixel-width pixel-height)
-					 "PNG:-")
+		    (cond ((or (< 0 (length pixel-width))
+			       (< 0 (length pixel-height)))
+			   (call-process-region (point-min) (point-max)
+						predoc-convert-program
+						t 
+						'(t nil)
+						nil
+						"-" 
+						"-resize"
+						(format "%sx%s" pixel-width pixel-height)
+						"PNG:-"))
+			  (t
+			   (call-process-region (point-min) (point-max)
+						predoc-convert-program
+						t 
+						'(t nil)
+						nil
+						"-" 
+						"PNG:-")))
 		    ;; (message (format "2: min = %s  max = %s " (point-min) (point-max)))
-		    (buffer-substring-no-properties (point-min) (point-max)))))))
-      (if data
-	  (create-image data 'png 'data :ascent 'center)
-	(create-image file nil nil :ascent 'center)))))
+		    (buffer-substring-no-properties (point-min) (point-max)))))
+	  (create-image data 'png 'data :ascent 'center))))))
 
 
 (defun predoc-insert-image-file (beg end attr-alist)
@@ -164,7 +195,7 @@ Buffer string between BEG and END are replaced with URL."
 	
 	;; get the image data file
 	(setq image (predoc-create-image
-		     (predoc-expand-full-path src)
+		     src
 		     width height))
 	;; push to cache
 	(push 
@@ -234,7 +265,7 @@ Buffer string between BEG and END are replaced with URL."
 	(_alink-elem-pattern
 	 "[>]\\([^<]+\\)[<][/][aA][>]")
 	(_pre-pattern
-	 "\\(<pre>\\|<PRE>\\|</pre>\\|</PRE>\\)")
+	 "\\(<[pP][rR][eE]\\|</pre>\\|</PRE>\\)")
 	(_image-pattern
  	 "\\(<img[ ]+\\)\\([^>]+\\)\\(>\\)"))
 
@@ -326,6 +357,7 @@ Buffer string between BEG and END are replaced with URL."
     (interactive)
     (let ((cur (point)))
       (cond
+       ;; <tag ...>
        ((equal ?< (char-after (point)))
 	(if (predoc-get-image (point))
 	    (progn
@@ -334,20 +366,48 @@ Buffer string between BEG and END are replaced with URL."
 	      (message "image"))
 	  (message"not image"))
 	(delete-char 1))
+       ;; &entity;
        ((equal ?& (char-after (point)))
 	(delete-char 1))
+       ;; entity;
        ((re-search-forward "[a-zA-Z]+;" (point-at-eol) t)
 	(goto-char cur)
 	(if (eq (match-beginning 0) cur)
 	    (progn
 	      (insert "&")
 	      (backward-char 1))))
+       ;; img src=...>   or   a href=...>
        ((re-search-forward "\\([aA][ ]+\\|[iI][mM][gG][ ]+\\)" (point-at-eol) t)
 	(goto-char cur)
 	(if (eq (match-beginning 0) cur)
 	    (progn
 	      (insert "<")
 	      (backward-char 1))))
+       ;; path/of/image/file.(jpg|png|bmp ... )
+       ((re-search-forward (concat "\\(.+\\)\\("
+				   (mapconcat
+				    (lambda (str) str)
+				    predoc-image-prefix-list
+				    "\\|")
+				   "\\)[\n\t ]")
+			   (+ (point-at-eol) 1)
+			   t)
+	(goto-char (match-end 2))
+	(insert (format "\" width=\"%d\"/> " predoc-insert-image-size))
+	(goto-char (match-beginning 1))
+	(let ((img-str "<img src=\""))
+	  (insert img-str)
+	  (backward-char (string-bytes img-str))))
+       ;; http://host/path/of/contents... anchor-keyword
+       ((re-search-forward "\\(http://[^\t ]+\\)[\t ]+\\([^\t ]+\\)[\n\t ]"
+			   (+ (point-at-eol) 1)
+			   t)
+	(let ((url (match-string 1))
+	      (str (match-string 2)))
+	  (delete-region (match-beginning 1) (match-end 2))
+	  (goto-char (match-beginning 1))
+	  (insert (format "<a href=\"%s\" />%s</a>" url str))))
+       ;; No match
        (t
 	(message "No match predoc tags.")))))
 
@@ -360,7 +420,7 @@ Buffer string between BEG and END are replaced with URL."
       (with-temp-buffer
 	(shell-command (mapconcat
 			(lambda (x)
-			  (format "find ./ -iname '*.%s'; " x))
+			  (format "find ./ -follow -iname '*.%s'; " x))
 			predoc-image-prefix-list
 			"")
 		       (current-buffer))
@@ -369,15 +429,16 @@ Buffer string between BEG and END are replaced with URL."
     
     (mapcar
      (lambda (x)
-       (insert
-	(format "<img src=\"%s\" width=\"%d\"/> " x predoc-insert-image-size)))
+       (when (string-match "^./" x)
+	 (insert
+	  (format "<img src=\"%s\" width=\"%d\"/> " x predoc-insert-image-size))))
      (search-image-files)))
 
   (when
       (save-excursion
 	(progn
 	  (goto-char (point-min))
-	  (re-search-forward "^[ ]*<pre>" (point-at-eol) t)))
+	  (re-search-forward "^[ ]*<[pP][rR][eE]" (point-at-eol) t)))
     
     (predoc-mode-hookfunc-stuff)
     
